(function() {

	/**
		comprizes of methods calculating the difference between dates;
		methods name reflects the result's unit. All the methods
		take two Date arguments - a start Date (from when to count), 
		and an end Date. An optional third argument defines 'toFixed'
		precision and defaults to 0. Returns Number.
	*/
	var oTimeApart = {
		/**
			returns how many full (1000ms) seconds separate the two dates;
		*/	
		second : function(oFrom, oTo) {
			return  +(( (oTo.getTime() - oFrom.getTime())/1000 ).toFixed(arguments[2]));
		},
		/**
			returns how many full (60sec) minutes separate the two dates;
		*/
		minute : function(oFrom, oTo) {
			return  +(( (oTo.getTime() - oFrom.getTime())/(60*1000) ).toFixed(arguments[2]));
		},
		/**
			returns how many full (60min) hours separate the two dates;
		*/
		hour : function(oFrom, oTo) {
			return  +(( (oTo.getTime() - oFrom.getTime())/(60*60*1000) ).toFixed(arguments[2]));
		},
		/**
			returns how many full (24h) days separate the two dates;
		*/
		day : function(oFrom, oTo) {
			return  +(( (oTo.getTime() - oFrom.getTime())/(24*60*60*1000) ).toFixed(arguments[2]));
		},
		/**
			returns how many calendar (not 24h) days separate the two dates,
			e.g. it returnes 1 for ( 2009-10-13 10:00pm, 2009-10-14 1.00am );
			The difference must be visible in hours. (Fix to milisec?)
		*/
		calendarDay : function(oFrom, oTo) {
			var iHours = this.hour(oFrom, oTo, 7);
			var iDays = iHours/24 >> 0;
			if (iHours > 0) {
				if ( oTo.getHours() < iHours - 24*iDays )
					++iDays;
			} else if (iHours < 0) {
				if ( 23 - oTo.getHours() < -(iHours - 24*iDays) )
					--iDays;
			}
			return iDays;
		},
		/**
			returns how many full (7 day) weeks separate the two dates;
		*/
		week : function(oFrom, oTo){
			return +(( (oTo.getTime() - oFrom.getTime())/(7*24*60*60*1000) ).toFixed(arguments[2]));
		},
		/**
			returns how many calendar (not 7 day) weeks separate 
			the two dates (a week starts with Sunday), e.g. it 
			returns 1 for ( Saturday the 11th, Sunday the 12th );
			The difference must be visible in days. (Fix to milisec?)
		*/
		calendarWeek : function(oFrom, oTo) {
			var iDays = this.calendarDay(oFrom, oTo);
			var iWeeks = iDays/7 >> 0;
			if (iDays > 0) {
				if ( oTo.getDay() < (iDays - 7*iWeeks) )
					++iWeeks;
			} else if (iDays < 0) {
				if ( 6 - oTo.getDay() < -(iDays - 7*iWeeks) )
					--iWeeks;
			}
			return iWeeks;
		},
		/**
			returns how many calendar (not ~30 day) months separate the two dates;
			e.g. it returns 1 for ( April 30th, May 1st )
		*/
		calendarMonth : function(oFrom, oTo) {
			return 12*this.calendarYear(oFrom, oTo) + (oTo.getMonth() - oFrom.getMonth())
		},
		/**
			returns how many full (365.2425 day) years separate the two dates;
		*/
		year : function(oFrom, oTo) {
			return +(( (oTo.getTime() - oFrom.getTime())/(365.2425*24*60*60*1000) ).toFixed(arguments[2]));
		},
		/**
			returns how many calendar (not 365 day) years separate the two dates,
			e.g. it returns 1 for ( 1410-12-31, 1411-01-01 )
		*/
		calendarYear : function(oFrom, oTo) {
			return oTo.getFullYear() - oFrom.getFullYear();
		}
	};
	
	/**
		returns String like: 14:40pm April 1st, 2009
	*/
	Date.prototype.humanReadable = function() {
		return this.oclock() +' '+this.getMonthName()+' '					// '12:00pm April '
			+this.getDate()+(['th','st','nd'][this.getDate()%10] || 'th')	// '27th'
			+', '+this.getFullYear();										// ', 1984'
	};
	
	Date.prototype.oclock = function() {
		var hour = this.getHours();
		var min = this.getMinutes();
		return (hour%12||12) + ':' + (min < 10 ? '0' : '') + min + ( hour<12 ? 'am' : 'pm' );
	};
	
	Date.prototype.getDayName = function() {
		return ['Sunday','Monday','Teusday','Wednesday','Thursday','Friday','Saturday'][this.getDay()];
	};
	
	Date.prototype.getMonthName = function() {
		return ['January','February','March','April','May','June','July','August','September','October','November','December'][this.getMonth()];
	};
		
		
	/**
		Returns the result of oTimeApart object method indicated by 'sUnit' (default is 'second'),
		called with this as oFrom Date and oNow (that defaults to new Date) as oTo Date;
	*/
	Date.prototype.elapsedBy = function(sUnit, oNow) {
		oNow = oNow || new Date();
		sUnit = sUnit || 'second';
		
		if (!(sUnit in oTimeApart) || !(oNow instanceof Date))
			throw new Error('Date.prototype.elapsedBy: invalid argument passed')
		
		return oTimeApart[sUnit](this,oNow);
	};

	
	function howMany(apart, period) {
		var s;

		if (apart == 0) {
			s = 'this '+period;
		} 
		else if (apart == 1 || !~apart) {
			s = ((!~apart)?'next ':'last ')+period;
		} 
		else {	//
			s = ((apart < 0)?-1:1)*apart + ' ' + period + 's';
			s = (apart < 0)?('in '+s):(s+' ago');
		}

		return s;
	};
	
	Date.prototype.happens = function(oNow) {
		
		oNow = (oNow instanceof Date)?oNow:(new Date);

		var daysApart = oTimeApart['day'](this, oNow);
		
		if (daysApart != 0 && daysApart > -7 && daysApart < 7) {
			var s;
			if (daysApart == 1 || !~daysApart) {
				s = ['tomorrow','yesterday'][~daysApart*(-0.5)] 
					+ ' at ' + this.oclock();;
			}
			else {
				s = ((daysApart < 0)?'next ':'last ') + this.getDayName() 
					+ ' at ' + this.oclock();
			} 
			return s;
		} 
		else if (daysApart == 0) {
			var periods = ['hour','minute','second'];
			for (var i = 0, period; period = periods[i]; ++i) {
				if (apart = oTimeApart[period](this, oNow)) {
					return howMany(apart, period);
				}
			}
			return 'right now!'
		} 
		else {
			var monthsApart = oTimeApart['calendarMonth'](this, oNow);
			var yearsApart = oTimeApart['calendarYear'](this, oNow);
			
			if (yearsApart && Math.abs(monthsApart) > 6) {
				return howMany(yearsApart, 'year');
			} 
			else if (monthsApart) {
				return howMany(monthsApart, 'month');
			} 
			else {
				var weeksApart = oTimeApart['calendarWeek'](this, oNow);
				return howMany(weeksApart, 'week');
			}
		}
	
	};

})();

